package main

import "os"
import "git.tebibyte.media/sashakoshka/fspl/cli"
import "git.tebibyte.media/sashakoshka/fspl/entity"
import "git.tebibyte.media/sashakoshka/fspl/compiler"
import ferrors "git.tebibyte.media/sashakoshka/fspl/errors"

func main () {
	compiler := new(compiler.Compiler)
	compiler.Stderr = os.Stderr
	
	help := cli.NewFlag (
		'h', "help",
		"Display usage information and exit")
	debug := cli.NewFlag (
		0, "debug",
		"Print extra debug information while compiling")
	quiet := cli.NewFlag (
		'q', "quiet",
		"Don't print warnings, errors, etc.")
	format := cli.NewInputFlag (
		'm', "format",
		"Output format (.s, .o, .ll)", "",
		cli.NewValSet(".s", ".o", ".ll"))
	output := cli.NewInputFlag (
		'o', "output",
		"Output filename", "",
		cli.ValString)
	optimization := cli.NewInputFlag (
		'O', "optimization",
		"Optimization level (0-3)", "0",
		cli.NewValSet("0", "1", "2", "3"))

	application := cli.New (
		"Compile FSPL source files",
		help,
		debug,
		quiet,
		format,
		output,
		optimization)
	
	application.Syntax = "[OPTION]... FILE"
	application.ParseOrExit(os.Args)
	if help.Value != "" {
		application.Usage()
		os.Exit(2)
	}
	if len(application.Args) != 1 {
		compiler.Errorln("please specify one unit address")
		application.Usage()
		os.Exit(2)
	}
	
	compiler.Output       = output.Value
	compiler.Optimization = optimization.Value
	compiler.Format       = format.Value
	compiler.Debug        = debug.Value != ""
	compiler.Quiet        = quiet.Value != ""

	err := compiler.CompileUnit(entity.Address(application.Args[0]))
	if err != nil {
		compiler.Errorln(ferrors.Format(err))
		os.Exit(1)
	}
}
